"""
This file contains any constants used in the docx-form module.
"""

XML_PREFIX = "{http://schemas.openxmlformats.org/wordprocessingml/2006/main}"
XML_CHECK = "{http://schemas.microsoft.com/office/word/2010/wordml}"
